'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = srcSetLoader;

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var DEFAULT_SIZE = 'default';

function removeResourceQuery(resource) {
  return resource.split('?')[0];
}

function splitRemainingRequest(remainingRequest) {
  var split = remainingRequest.split('!');
  var rawResource = split.pop();
  var resource = removeResourceQuery(rawResource);
  return [split, resource];
}

function rebuildRemainingRequest(loaders, resource) {
  return '-!' + [].concat(_toConsumableArray(loaders), [resource]).join('!');
}

function buildResizeLoader(rawSize) {
  var size = parseInt(rawSize, 10);

  return _url2.default.format({
    pathname: _path2.default.join(__dirname, './resize-loader'),
    query: { size: size }
  });
}

function createResizeRequest(size, existingLoaders, resource) {
  var loaders = size === DEFAULT_SIZE ? existingLoaders : [].concat(_toConsumableArray(existingLoaders), [buildResizeLoader(size)]);

  var remainingRequest = rebuildRemainingRequest(loaders, resource);
  return 'require(' + JSON.stringify(remainingRequest) + ')';
}

function createPlaceholderRequest(resource, size, lightweight) {
  var loaderOptions = {
    pathname: _path2.default.join(__dirname, './placeholder-loader'),
    query: {
      lightweight: lightweight
    }
  };

  var actualSize = toNumber(size);
  if (!Number.isNaN(actualSize)) {
    loaderOptions.query.size = actualSize;
  }

  return 'require(\'!!' + _url2.default.format(loaderOptions) + '!' + resource + '\')';
}

function toNumber(item) {
  if (typeof item !== 'string') {
    return Number.NaN;
  }

  return Number(item);
}

function forEach(items, cb) {
  if (Array.isArray(items)) {
    return items.forEach(cb);
  }

  return cb(items);
}

function buildSources(sizes, loaders, resource) {
  var sources = {};

  forEach(sizes, function (size) {
    if (size != null && size !== DEFAULT_SIZE && !/\d+w/.test(size)) {
      throw new TypeError('srcset-loader: Received size "' + size + '" does not match the format "\\d+w" nor "' + DEFAULT_SIZE + '"');
    }

    var actualSize = size || DEFAULT_SIZE;
    sources[actualSize] = createResizeRequest(actualSize, loaders, resource(size));
  });

  return sources;
}

function stringifySources(sources) {
  return '\n{\n  ' + Object.keys(sources).map(function (source) {
    return '"' + source + '": ' + sources[source];
  }).join(',\n') + '\n}\n';
}

function stringifySrcSet(sources) {
  return Object.keys(sources).map(function (size) {
    if (size === 'default') {
      return '' + sources[size];
    }

    return sources[size] + ' + " ' + size + '"';
  }).join('+","+');
}

function getSizes(sizes) {
  if (sizes == null || Array.isArray(sizes)) {
    return sizes;
  }

  if (typeof sizes === 'string') {
    return sizes.split('+');
  }

  throw new TypeError('srcset-loader: "?sizes=' + sizes + '" is invalid - expected a query like "?sizes[]=<size>&sizes[]=..." or "?sizes=<size>+<size>+...".');
}

function isLightweight(loaderQuery, resourceQuery) {
  if (loaderQuery.lightweight !== undefined) {
    return loaderQuery.lightweight;
  }

  if (resourceQuery.lightweight !== undefined) {
    return resourceQuery.lightweight;
  }

  return false;
}

function srcSetLoader(content) {
  return content;
}

srcSetLoader.pitch = function srcSetLoaderPitch(remainingRequest) {
  var loaderQuery = (0, _util.parseQuery)(this.query);
  var resourceQuery = (0, _util.parseQuery)(this.resourceQuery);

  var lightweight = isLightweight(loaderQuery, resourceQuery);
  if (typeof lightweight !== 'boolean') {
    throw new TypeError('srcset-loader: "?lightweight=' + lightweight + '" is invalid - expected a boolean.');
  }

  // check it isn't undefined so the resource can disable the loader configuration with `false`.
  var placeholder = resourceQuery.placeholder !== undefined ? resourceQuery.placeholder : loaderQuery.placeholder;

  // sizes can be falsy,
  // it will just return the original image along with the placeholder if requested.
  var sizes = getSizes(resourceQuery.sizes !== undefined ? resourceQuery.sizes : loaderQuery.sizes);

  // neither is requested, no need to run this loader.
  if (!placeholder && !sizes) {
    return undefined;
  }

  var _splitRemainingReques = splitRemainingRequest(remainingRequest),
      _splitRemainingReques2 = _slicedToArray(_splitRemainingReques, 2),
      loaders = _splitRemainingReques2[0],
      resource = _splitRemainingReques2[1];

  var transformResource = loaderQuery.transformResource || function (resource, size) {
    return resource + '?size=' + size;
  };

  var sources = buildSources(sizes, loaders, function (size) {
    return transformResource(resource, size);
  });

  var srcSet = !lightweight ? 'srcSet: ' + stringifySrcSet(sources) + ',' : '';

  var placeholderScript = placeholder ? 'placeholder: ' + createPlaceholderRequest(resource, placeholder, lightweight) + ',' : '';

  return '\nmodule.exports = {\n  sources: ' + stringifySources(sources) + ',\n  ' + srcSet + '\n  ' + placeholderScript + '\n};\n';
};

// webpack pitch loaders expect commonJS
module.exports.pitch = srcSetLoader.pitch;